﻿using SMTPServer.Exceptions;
using System;
using System.Collections.Generic;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.Threading;

namespace SMTPServer
{
    public delegate void ConnectedEventHandler(object source, ConnectedEventArgs e);

    public class ConnectedEventArgs : EventArgs
    {
        private string EventInfo;

        public Socket Client { get; private set; }

        public ConnectedEventArgs(Socket client)
        {
            Client = client;
        }
    }

    public class TCPListener
    {
        static List<int> _Ports = null;
        TcpListener _Listener;

        public event ConnectedEventHandler OnConnected;

        public TCPListener(int port)
        {
            if(_Ports == null)
            {
                _Ports = new List<int>();
            }

            foreach(var i in _Ports)
            {
                if (i == port)
                {
                    throw new PortUsedException();
                }
            }

            _Ports.Add(port);
            StartListening(new TcpListener(IPAddress.Any, port));
        }

        private void StartListening(TcpListener listener)
        {
            listener.Start();
            _Listener = listener;
            Thread thread = new Thread(new ThreadStart(ListenerAsync));
        }

        private async void ListenerAsync()
        {
            var client = await _Listener.AcceptSocketAsync();
            OnConnected(this, new ConnectedEventArgs(client));
        }
    }
}
